@@ -201,6 +201,7 @@ impl ZellijPlugin for State {
         subscribe(&[
             EventType::ModeUpdate,
             EventType::TabUpdate,
+            EventType::PaneUpdate,
             EventType::CopyToClipboard,
             EventType::InputReceived,
             EventType::SystemClipboardFailure,
@@ -699,10 +700,6 @@ pub mod tests {
         );
     }
 
-    fn get_palette() -> Palette {
-        Palette::default()
-    }
-
     #[test]
     fn style_key_with_modifier_only_chars() {
         let keyvec = vec![
