@@ -7,18 +7,19 @@ mod keyboard_parser;
 pub mod old_config_converter;
 mod stdin_ansi_parser;
 mod stdin_handler;
+#[cfg(feature = "web_server_capability")]
+pub mod web_client;
 
 use log::info;
 use std::env::current_exe;
 use std::io::{self, Write};
+use std::net::{IpAddr, Ipv4Addr};
 use std::path::Path;
 use std::process::Command;
 use std::sync::{Arc, Mutex};
 use std::thread;
 use zellij_utils::errors::FatalError;
-
-use notify_debouncer_full::notify::{self, Event, RecursiveMode, Watcher};
-use zellij_utils::setup::Setup;
+use zellij_utils::shared::web_server_base_url;
 
 use crate::stdin_ansi_parser::{AnsiStdinInstruction, StdinAnsiParser, SyncOutput};
 use crate::{
@@ -32,7 +33,10 @@ use zellij_utils::{
     data::{ClientId, ConnectToSession, KeyWithModifier, Style},
     envs,
     errors::{ClientContext, ContextType, ErrorInstruction},
-    input::{config::Config, options::Options},
+    input::{
+        config::{watch_config_file_changes, Config},
+        options::Options,
+    },
     ipc::{ClientAttributes, ClientToServerMsg, ExitReason, ServerToClientMsg},
     pane_size::Size,
 };
@@ -55,7 +59,12 @@ pub(crate) enum ClientInstruction {
     UnblockCliPipeInput(()), // String -> pipe name
     CliPipeOutput((), ()),   // String -> pipe name, String -> output
     QueryTerminalSize,
-    WriteConfigToDisk { config: String },
+    WriteConfigToDisk {
+        config: String,
+    },
+    StartWebServer,
+    #[allow(dead_code)] // we need the session name here even though we're not currently using it
+    RenamedSession(String), // String -> new session name
 }
 
 impl From<ServerToClientMsg> for ClientInstruction {
@@ -80,6 +89,8 @@ impl From<ServerToClientMsg> for ClientInstruction {
             ServerToClientMsg::WriteConfigToDisk { config } => {
                 ClientInstruction::WriteConfigToDisk { config }
             },
+            ServerToClientMsg::StartWebServer => ClientInstruction::StartWebServer,
+            ServerToClientMsg::RenamedSession(name) => ClientInstruction::RenamedSession(name),
         }
     }
 }
@@ -102,6 +113,8 @@ impl From<&ClientInstruction> for ClientContext {
             ClientInstruction::CliPipeOutput(..) => ClientContext::CliPipeOutput,
             ClientInstruction::QueryTerminalSize => ClientContext::QueryTerminalSize,
             ClientInstruction::WriteConfigToDisk { .. } => ClientContext::WriteConfigToDisk,
+            ClientInstruction::StartWebServer => ClientContext::StartWebServer,
+            ClientInstruction::RenamedSession(..) => ClientContext::RenamedSession,
         }
     }
 }
@@ -112,7 +125,46 @@ impl ErrorInstruction for ClientInstruction {
     }
 }
 
-fn spawn_server(socket_path: &Path, debug: bool) -> io::Result<()> {
+#[cfg(feature = "web_server_capability")]
+fn spawn_web_server(opts: &CliArgs) -> Result<String, String> {
+    let mut cmd = Command::new(current_exe().map_err(|e| e.to_string())?);
+    if let Some(config_file_path) = Config::config_file_path(opts) {
+        let config_file_path_exists = Path::new(&config_file_path).exists();
+        if !config_file_path_exists {
+            return Err(format!(
+                "Config file: {} does not exist",
+                config_file_path.display()
+            ));
+        }
+        // this is so that if Zellij itself was started with a different config file, we'll use it
+        // to start the webserver
+        cmd.arg("--config");
+        cmd.arg(format!("{}", config_file_path.display()));
+    }
+    cmd.arg("web");
+    cmd.arg("-d");
+    let output = cmd.output();
+    match output {
+        Ok(output) => {
+            if output.status.success() {
+                Ok(String::from_utf8_lossy(&output.stdout).to_string())
+            } else {
+                Err(String::from_utf8_lossy(&output.stderr).to_string())
+            }
+        },
+        Err(e) => Err(e.to_string()),
+    }
+}
+
+#[cfg(not(feature = "web_server_capability"))]
+fn spawn_web_server(_opts: &CliArgs) -> Result<String, String> {
+    log::error!(
+        "This version of Zellij was compiled without web server support, cannot run web server!"
+    );
+    Ok("".to_owned())
+}
+
+pub fn spawn_server(socket_path: &Path, debug: bool) -> io::Result<()> {
     let mut cmd = Command::new(current_exe()?);
     cmd.arg("--server");
     cmd.arg(socket_path);
@@ -171,6 +223,7 @@ pub fn start_client(
     pane_id_to_focus: Option<(u32, bool)>, // (pane_id, is_plugin)
     is_a_reconnect: bool,
     start_detached_and_exit: bool,
+    layout_is_welcome_screen: bool,
 ) -> Option<ConnectToSession> {
     if start_detached_and_exit {
         start_server_detached(os_input, opts, config, config_options, info, layout);
@@ -182,6 +235,7 @@ pub fn start_client(
         .support_kitty_keyboard_protocol
         .map(|e| !e)
         .unwrap_or(false);
+    let should_start_web_server = config_options.web_server.map(|w| w).unwrap_or(false);
     let mut reconnect_to_session = None;
     let clear_client_terminal_attributes = "\u{1b}[?1l\u{1b}=\u{1b}[r\u{1b}[?1000l\u{1b}[?1002l\u{1b}[?1003l\u{1b}[?1005l\u{1b}[?1006l\u{1b}[?12l";
     let take_snapshot = "\u{1b}[?1049h";
@@ -224,6 +278,13 @@ pub fn start_client(
             hide_session_name: config.ui.pane_frames.hide_session_name,
         },
     };
+    let web_server_ip = config_options
+        .web_server_ip
+        .unwrap_or_else(|| IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)));
+    let web_server_port = config_options.web_server_port.unwrap_or_else(|| 8082);
+    let has_certificate =
+        config_options.web_server_cert.is_some() && config_options.web_server_key.is_some();
+    let enforce_https_for_localhost = config_options.enforce_https_for_localhost.unwrap_or(false);
 
     let create_ipc_pipe = || -> std::path::PathBuf {
         let mut sock_dir = ZELLIJ_SOCK_DIR.clone();
@@ -238,6 +299,7 @@ pub fn start_client(
             envs::set_session_name(name.clone());
             os_input.update_session_name(name);
             let ipc_pipe = create_ipc_pipe();
+            let is_web_client = false;
 
             (
                 ClientToServerMsg::AttachClient(
@@ -246,6 +308,7 @@ pub fn start_client(
                     config_options.clone(),
                     tab_position_to_focus,
                     pane_id_to_focus,
+                    is_web_client,
                 ),
                 ipc_pipe,
             )
@@ -256,6 +319,12 @@ pub fn start_client(
             let ipc_pipe = create_ipc_pipe();
 
             spawn_server(&*ipc_pipe, opts.debug).unwrap();
+            if should_start_web_server {
+                if let Err(e) = spawn_web_server(&opts) {
+                    log::error!("Failed to start web server: {}", e);
+                }
+            }
+
             let successfully_written_config =
                 Config::write_config_to_disk_if_it_does_not_exist(config.to_string(true), &opts);
             // if we successfully wrote the config to disk, it means two things:
@@ -265,6 +334,7 @@ pub fn start_client(
             // If these two are true, we should launch the setup wizard, if even one of them is
             // false, we should never launch it.
             let should_launch_setup_wizard = successfully_written_config;
+            let is_web_client = false;
 
             (
                 ClientToServerMsg::NewClient(
@@ -274,7 +344,9 @@ pub fn start_client(
                     Box::new(config_options.clone()),
                     Box::new(layout.unwrap()),
                     Box::new(config.plugins.clone()),
+                    is_web_client,
                     should_launch_setup_wizard,
+                    layout_is_welcome_screen,
                 ),
                 ipc_pipe,
             )
@@ -358,11 +430,7 @@ pub fn start_client(
             let os_input = os_input.clone();
             let opts = opts.clone();
             move || {
-                // we keep the config_file_watcher here so that it is only dropped when this thread
-                // exits (which is when the client disconnects/detaches), once it's dropped it
-                // stops watching and we want it to keep watching the config file path for changes
-                // as long as the client is alive
-                let _config_file_watcher = report_changes_in_config_file(&opts, &os_input);
+                report_changes_in_config_file(&opts, &os_input);
                 os_input.handle_signals(
                     Box::new({
                         let os_api = os_input.clone();
@@ -561,6 +629,26 @@ pub fn start_client(
                     },
                 }
             },
+            ClientInstruction::StartWebServer => {
+                let web_server_base_url = web_server_base_url(
+                    web_server_ip,
+                    web_server_port,
+                    has_certificate,
+                    enforce_https_for_localhost,
+                );
+                match spawn_web_server(&opts) {
+                    Ok(_) => {
+                        let _ = os_input.send_to_server(ClientToServerMsg::WebServerStarted(
+                            web_server_base_url,
+                        ));
+                    },
+                    Err(e) => {
+                        log::error!("Failed to start web_server: {}", e);
+                        let _ =
+                            os_input.send_to_server(ClientToServerMsg::FailedToStartWebServer(e));
+                    },
+                }
+            },
             _ => {},
         }
     }
@@ -611,6 +699,8 @@ pub fn start_server_detached(
     envs::set_zellij("0".to_string());
     config.env.set_vars();
 
+    let should_start_web_server = config_options.web_server.map(|w| w).unwrap_or(false);
+
     let palette = config
         .theme_config(config_options.theme.as_ref())
         .unwrap_or_else(|| os_input.load_palette().into());
@@ -640,8 +730,14 @@ pub fn start_server_detached(
             let ipc_pipe = create_ipc_pipe();
 
             spawn_server(&*ipc_pipe, opts.debug).unwrap();
+            if should_start_web_server {
+                if let Err(e) = spawn_web_server(&opts) {
+                    log::error!("Failed to start web server: {}", e);
+                }
+            }
             let should_launch_setup_wizard = false; // no setup wizard when starting a detached
                                                     // server
+            let is_web_client = false;
 
             (
                 ClientToServerMsg::NewClient(
@@ -651,7 +747,9 @@ pub fn start_server_detached(
                     Box::new(config_options.clone()),
                     Box::new(layout.unwrap()),
                     Box::new(config.plugins.clone()),
+                    is_web_client,
                     should_launch_setup_wizard,
+                    false,
                 ),
                 ipc_pipe,
             )
@@ -666,59 +764,20 @@ pub fn start_server_detached(
     os_input.send_to_server(first_msg);
 }
 
-fn report_changes_in_config_file(
-    opts: &CliArgs,
-    os_input: &Box<dyn ClientOsApi>,
-) -> Option<Box<dyn Watcher>> {
-    match Config::config_file_path(&opts) {
-        Some(config_file_path) => {
-            let mut watcher = notify::recommended_watcher({
-                let os_input = os_input.clone();
-                let opts = opts.clone();
-                let config_file_path = config_file_path.clone();
-                move |res: Result<Event, _>| match res {
-                    Ok(event)
-                        if (event.kind.is_create() || event.kind.is_modify())
-                            && event.paths.contains(&config_file_path) =>
-                    {
-                        match Setup::from_cli_args(&opts) {
-                            Ok((
-                                new_config,
-                                _layout,
-                                _config_options,
-                                _config_without_layout,
-                                _config_options_without_layout,
-                            )) => {
-                                os_input.send_to_server(ClientToServerMsg::ConfigWrittenToDisk(
-                                    new_config,
-                                ));
-                            },
-                            Err(e) => {
-                                log::error!("Failed to reload config: {}", e);
-                            },
-                        }
-                    },
-                    Err(e) => log::error!("watch error: {:?}", e),
-                    _ => {},
-                }
-            })
-            .unwrap();
-            if let Some(config_file_parent_folder) = config_file_path.parent() {
-                match watcher.watch(&config_file_parent_folder, RecursiveMode::Recursive) {
-                    Ok(_) => Some(Box::new(watcher)),
-                    Err(e) => {
-                        log::error!("Failed to watch config file folder: {}", e);
-                        None
-                    },
-                }
-            } else {
-                log::error!("Could not find config parent folder");
-                None
-            }
-        },
-        None => {
-            log::error!("Failed to find config path");
-            None
-        },
+pub fn report_changes_in_config_file(opts: &CliArgs, os_input: &Box<dyn ClientOsApi>) {
+    if let Some(config_file_path) = Config::config_file_path(&opts) {
+        let os_input = os_input.clone();
+        std::thread::spawn(move || {
+            let rt = tokio::runtime::Runtime::new().unwrap();
+            rt.block_on(async move {
+                watch_config_file_changes(config_file_path, move |new_config| {
+                    let os_input = os_input.clone();
+                    async move {
+                        os_input.send_to_server(ClientToServerMsg::ConfigWrittenToDisk(new_config));
+                    }
+                })
+                .await;
+            });
+        });
     }
 }
