@@ -25,8 +25,9 @@ use wasm_bridge::WasmBridge;
 use async_std::{channel, future::timeout, task};
 use zellij_utils::{
     data::{
-        ClientInfo, Event, EventType, InputMode, MessageToPlugin, PermissionStatus, PermissionType,
-        PipeMessage, PipeSource, PluginCapabilities,
+        ClientInfo, Event, EventType, FloatingPaneCoordinates, InputMode, MessageToPlugin,
+        PermissionStatus, PermissionType, PipeMessage, PipeSource, PluginCapabilities,
+        WebServerStatus,
     },
     errors::{prelude::*, ContextType, PluginContext},
     input::{
@@ -53,8 +54,11 @@ pub enum PluginInstruction {
         Option<PaneId>, // pane id to replace if this is to be opened "in-place"
         ClientId,
         Size,
-        Option<PathBuf>, // cwd
-        bool,            // skip cache
+        Option<PathBuf>,  // cwd
+        Option<PluginId>, // the focused plugin id if relevant
+        bool,             // skip cache
+        Option<bool>,     // should focus plugin
+        Option<FloatingPaneCoordinates>,
     ),
     LoadBackgroundPlugin(RunPluginOrAlias, ClientId),
     Update(Vec<(Option<PluginId>, Option<ClientId>, Event)>), // Focused plugin / broadcast, client_id, event data
@@ -75,9 +79,9 @@ pub enum PluginInstruction {
         Option<TerminalAction>,
         Option<TiledPaneLayout>,
         Vec<FloatingPaneLayout>,
-        usize, // tab_index
-        bool,  // should change focus to new tab
-        ClientId,
+        usize,            // tab_index
+        bool,             // should change focus to new tab
+        (ClientId, bool), // bool -> is_web_client
     ),
     ApplyCachedEvents {
         plugin_ids: Vec<PluginId>,
@@ -160,6 +164,8 @@ pub enum PluginInstruction {
     WatchFilesystem,
     ListClientsToPlugin(SessionLayoutMetadata, PluginId, ClientId),
     ChangePluginHostDir(PathBuf, PluginId, ClientId),
+    WebServerStarted(String), // String -> the base url of the web server
+    FailedToStartWebServer(String),
     Exit,
 }
 
@@ -207,6 +213,8 @@ impl From<&PluginInstruction> for PluginContext {
             },
             PluginInstruction::ListClientsToPlugin(..) => PluginContext::ListClientsToPlugin,
             PluginInstruction::ChangePluginHostDir(..) => PluginContext::ChangePluginHostDir,
+            PluginInstruction::WebServerStarted(..) => PluginContext::WebServerStarted,
+            PluginInstruction::FailedToStartWebServer(..) => PluginContext::FailedToStartWebServer,
         }
     }
 }
@@ -246,7 +254,7 @@ pub(crate) fn plugin_thread_main(
         engine,
         plugin_dir,
         path_to_default_shell,
-        zellij_cwd,
+        zellij_cwd.clone(),
         capabilities,
         client_attributes,
         default_shell,
@@ -280,10 +288,19 @@ pub(crate) fn plugin_thread_main(
                 client_id,
                 size,
                 cwd,
+                focused_plugin_id,
                 skip_cache,
+                should_focus_plugin,
+                floating_pane_coordinates,
             ) => {
                 run_plugin_or_alias.populate_run_plugin_if_needed(&plugin_aliases);
-                let cwd = run_plugin_or_alias.get_initial_cwd().or(cwd);
+                let cwd = run_plugin_or_alias.get_initial_cwd().or(cwd).or_else(|| {
+                    if let Some(plugin_id) = focused_plugin_id {
+                        wasm_bridge.get_plugin_cwd(plugin_id, client_id)
+                    } else {
+                        None
+                    }
+                });
                 let run_plugin = run_plugin_or_alias.get_run_plugin();
                 let start_suppressed = false;
                 match wasm_bridge.load_plugin(
@@ -304,10 +321,17 @@ pub(crate) fn plugin_thread_main(
                             tab_index,
                             plugin_id,
                             pane_id_to_replace,
-                            cwd,
+                            cwd.clone(),
                             start_suppressed,
+                            floating_pane_coordinates,
+                            should_focus_plugin,
                             Some(client_id),
                         )));
+
+                        drop(bus.senders.send_to_pty(PtyInstruction::ReportPluginCwd(
+                            plugin_id,
+                            cwd.unwrap_or_else(|| zellij_cwd.clone()),
+                        )));
                     },
                     Err(e) => {
                         log::error!("Failed to load plugin: {e}");
@@ -378,6 +402,8 @@ pub(crate) fn plugin_thread_main(
                                                     None,
                                                     start_suppressed,
                                                     None,
+                                                    None,
+                                                    None,
                                                 ),
                                             ));
                                         },
@@ -416,7 +442,7 @@ pub(crate) fn plugin_thread_main(
                 mut floating_panes_layout,
                 tab_index,
                 should_change_focus_to_new_tab,
-                client_id,
+                (client_id, is_web_client),
             ) => {
                 // prefer connected clients so as to avoid opening plugins in the background for
                 // CLI clients unless no-one else is connected
@@ -488,7 +514,7 @@ pub(crate) fn plugin_thread_main(
                     tab_index,
                     plugin_ids,
                     should_change_focus_to_new_tab,
-                    client_id,
+                    (client_id, is_web_client),
                 )));
             },
             PluginInstruction::ApplyCachedEvents {
@@ -667,6 +693,7 @@ pub(crate) fn plugin_thread_main(
             } => {
                 let should_float = floating.unwrap_or(true);
                 let mut pipe_messages = vec![];
+                let floating_pane_coordinates = None; // TODO: do we want to allow this?
                 match plugin {
                     Some(plugin_url) => {
                         // send to specific plugin(s)
@@ -687,6 +714,8 @@ pub(crate) fn plugin_thread_main(
                             &bus,
                             &mut wasm_bridge,
                             &plugin_aliases,
+                            floating_pane_coordinates,
+                            None,
                         );
                     },
                     None => {
@@ -719,6 +748,7 @@ pub(crate) fn plugin_thread_main(
             } => {
                 let should_float = floating.unwrap_or(true);
                 let mut pipe_messages = vec![];
+                let floating_pane_coordinates = None; // TODO: do we want to allow this?
                 if let Some((plugin_id, client_id)) = plugin_and_client_id {
                     let is_private = true;
                     pipe_messages.push((
@@ -747,6 +777,8 @@ pub(crate) fn plugin_thread_main(
                                 &bus,
                                 &mut wasm_bridge,
                                 &plugin_aliases,
+                                floating_pane_coordinates,
+                                None,
                             );
                         },
                         None => {
@@ -790,6 +822,7 @@ pub(crate) fn plugin_thread_main(
                     .new_plugin_args
                     .as_ref()
                     .and_then(|n| n.pane_id_to_replace);
+                let floating_pane_coordinates = message.floating_pane_coordinates;
                 match (message.plugin_url, message.destination_plugin_id) {
                     (Some(plugin_url), None) => {
                         // send to specific plugin(s)
@@ -810,6 +843,8 @@ pub(crate) fn plugin_thread_main(
                             &bus,
                             &mut wasm_bridge,
                             &plugin_aliases,
+                            floating_pane_coordinates,
+                            message.new_plugin_args.and_then(|n| n.should_focus),
                         );
                     },
                     (None, Some(destination_plugin_id)) => {
@@ -897,8 +932,33 @@ pub(crate) fn plugin_thread_main(
                 wasm_bridge.start_fs_watcher_if_not_started();
             },
             PluginInstruction::ChangePluginHostDir(new_host_folder, plugin_id, client_id) => {
+                if let Ok(_) = wasm_bridge.change_plugin_host_dir(
+                    new_host_folder.clone(),
+                    plugin_id,
+                    client_id,
+                ) {
+                    drop(
+                        bus.senders.send_to_pty(PtyInstruction::ReportPluginCwd(
+                            plugin_id,
+                            new_host_folder,
+                        )),
+                    );
+                }
+            },
+            PluginInstruction::WebServerStarted(base_url) => {
+                let updates = vec![(
+                    None,
+                    None,
+                    Event::WebServerStatus(WebServerStatus::Online(base_url)),
+                )];
                 wasm_bridge
-                    .change_plugin_host_dir(new_host_folder, plugin_id, client_id)
+                    .update_plugins(updates, shutdown_send.clone())
+                    .non_fatal();
+            },
+            PluginInstruction::FailedToStartWebServer(error) => {
+                let updates = vec![(None, None, Event::FailedToStartWebServer(error))];
+                wasm_bridge
+                    .update_plugins(updates, shutdown_send.clone())
                     .non_fatal();
             },
             PluginInstruction::Exit => {
@@ -989,6 +1049,8 @@ fn pipe_to_specific_plugins(
     bus: &Bus<PluginInstruction>,
     wasm_bridge: &mut WasmBridge,
     plugin_aliases: &PluginAliases,
+    floating_pane_coordinates: Option<FloatingPaneCoordinates>,
+    should_focus: Option<bool>,
 ) {
     let is_private = true;
     let size = Size::default();
@@ -1010,6 +1072,8 @@ fn pipe_to_specific_plugins(
                 pane_title.clone(),
                 pane_id_to_replace.clone(),
                 cli_client_id,
+                floating_pane_coordinates,
+                should_focus.unwrap_or(false),
             );
             for (plugin_id, client_id) in all_plugin_ids {
                 pipe_messages.push((
@@ -1070,7 +1134,8 @@ fn load_background_plugin(
                 pane_id_to_replace,
                 cwd,
                 start_suppressed,
-                // None,
+                None,
+                None,
                 Some(client_id),
             )));
         },
