@@ -21,7 +21,9 @@ use std::{
 };
 use wasmtime::{Engine, Module};
 use zellij_utils::consts::{ZELLIJ_CACHE_DIR, ZELLIJ_TMP_DIR};
-use zellij_utils::data::{InputMode, PermissionStatus, PermissionType, PipeMessage, PipeSource};
+use zellij_utils::data::{
+    FloatingPaneCoordinates, InputMode, PermissionStatus, PermissionType, PipeMessage, PipeSource,
+};
 use zellij_utils::downloader::Downloader;
 use zellij_utils::input::keybinds::Keybinds;
 use zellij_utils::input::permission::PermissionCache;
@@ -318,14 +320,52 @@ impl WasmBridge {
     pub fn unload_plugin(&mut self, pid: PluginId) -> Result<()> {
         info!("Bye from plugin {}", &pid);
         let mut plugin_map = self.plugin_map.lock().unwrap();
-        for (running_plugin, _, workers) in plugin_map.remove_plugins(pid) {
+        for ((plugin_id, client_id), (running_plugin, subscriptions, workers)) in
+            plugin_map.remove_plugins(pid)
+        {
             for (_worker_name, worker_sender) in workers {
                 drop(worker_sender.send(MessageToWorker::Exit));
             }
-            let running_plugin = running_plugin.lock().unwrap();
-            let cache_dir = running_plugin.store.data().plugin_own_data_dir.clone();
-            if let Err(e) = std::fs::remove_dir_all(cache_dir) {
-                log::error!("Failed to remove cache dir for plugin: {:?}", e);
+            {
+                // if the plugin was intercepting key presses and for some reason did not clear
+                // this state, we make sure to do it ourselves so that the user will not get stuck
+                if running_plugin.lock().unwrap().intercepting_key_presses() {
+                    let _ = self
+                        .senders
+                        .send_to_screen(ScreenInstruction::ClearKeyPressesIntercepts(client_id));
+                }
+            }
+            let subscriptions = subscriptions.lock().unwrap();
+            if subscriptions.contains(&EventType::BeforeClose) {
+                let mut running_plugin = running_plugin.lock().unwrap();
+                match apply_before_close_event_to_plugin(
+                    pid,
+                    client_id,
+                    &mut running_plugin,
+                    self.senders.clone(),
+                ) {
+                    Ok(()) => {},
+                    Err(e) => {
+                        log::error!("{:?}", e);
+
+                        // https://stackoverflow.com/questions/66450942/in-rust-is-there-a-way-to-make-literal-newlines-in-r-using-windows-c
+                        let stringified_error = format!("{:?}", e).replace("\n", "\n\r");
+
+                        handle_plugin_crash(plugin_id, stringified_error, self.senders.clone());
+                    },
+                }
+                let cache_dir = running_plugin.store.data().plugin_own_data_dir.clone();
+                if let Err(e) = std::fs::remove_dir_all(cache_dir) {
+                    log::error!("Failed to remove cache dir for plugin: {:?}", e);
+                }
+            } else {
+                // this is duplicated because of locking/unlocking order between running_plugin and
+                // subscriptions
+                let running_plugin = running_plugin.lock().unwrap();
+                let cache_dir = running_plugin.store.data().plugin_own_data_dir.clone();
+                if let Err(e) = std::fs::remove_dir_all(cache_dir) {
+                    log::error!("Failed to remove cache dir for plugin: {:?}", e);
+                }
             }
         }
         self.cached_plugin_map.clear();
@@ -735,6 +775,27 @@ impl WasmBridge {
         }
         Ok(())
     }
+    pub fn get_plugin_cwd(&self, plugin_id: PluginId, client_id: ClientId) -> Option<PathBuf> {
+        self.plugin_map
+            .lock()
+            .unwrap()
+            .running_plugins()
+            .iter()
+            .find_map(|(p_id, c_id, running_plugin)| {
+                if p_id == &plugin_id && c_id == &client_id {
+                    let plugin_cwd = running_plugin
+                        .lock()
+                        .unwrap()
+                        .store
+                        .data()
+                        .plugin_cwd
+                        .clone();
+                    Some(plugin_cwd)
+                } else {
+                    None
+                }
+            })
+    }
     pub fn change_plugin_host_dir(
         &mut self,
         new_host_dir: PathBuf,
@@ -753,12 +814,6 @@ impl WasmBridge {
             .running_plugins_and_subscriptions()
             .iter()
             .cloned()
-            .filter(|(plugin_id, _client_id, _running_plugin, _subscriptions)| {
-                // TODO: cache this somehow in this case...
-                !&self
-                    .cached_events_for_pending_plugins
-                    .contains_key(&plugin_id)
-            })
             .collect();
         task::spawn({
             let senders = self.senders.clone();
@@ -1371,6 +1426,8 @@ impl WasmBridge {
         pane_title: Option<String>,
         pane_id_to_replace: Option<PaneId>,
         cli_client_id: Option<ClientId>,
+        floating_pane_coordinates: Option<FloatingPaneCoordinates>,
+        should_focus: bool,
     ) -> Vec<(PluginId, Option<ClientId>)> {
         let run_plugin = run_plugin_or_alias.get_run_plugin();
         match run_plugin {
@@ -1407,6 +1464,8 @@ impl WasmBridge {
                                 pane_id_to_replace,
                                 cwd,
                                 start_suppressed,
+                                floating_pane_coordinates,
+                                Some(should_focus),
                                 Some(client_id),
                             )));
                             vec![(plugin_id, Some(client_id))]
@@ -1543,6 +1602,7 @@ fn check_event_permission(
         | Event::FailedToWriteConfigToDisk(..)
         | Event::CommandPaneReRun(..)
         | Event::InputReceived => PermissionType::ReadApplicationState,
+        Event::WebServerStatus(..) => PermissionType::StartWebServer,
         _ => return (PermissionStatus::Granted, None),
     };
 
@@ -1651,3 +1711,36 @@ pub fn handle_plugin_crash(plugin_id: PluginId, message: String, senders: Thread
         loading_indication,
     ));
 }
+
+pub fn apply_before_close_event_to_plugin(
+    plugin_id: PluginId,
+    client_id: ClientId,
+    running_plugin: &mut RunningPlugin,
+    senders: ThreadSenders,
+) -> Result<()> {
+    let instance = &running_plugin.instance;
+
+    let err_context = || format!("Failed to apply event to plugin {plugin_id}");
+    let event = Event::BeforeClose;
+    let protobuf_event: ProtobufEvent = event
+        .clone()
+        .try_into()
+        .map_err(|e| anyhow!("Failed to convert to protobuf: {:?}", e))?;
+    let update = instance
+        .get_typed_func::<(), i32>(&mut running_plugin.store, "update")
+        .with_context(err_context)?;
+    wasi_write_object(running_plugin.store.data(), &protobuf_event.encode_to_vec())
+        .with_context(err_context)?;
+    let _should_render = update
+        .call(&mut running_plugin.store, ())
+        .with_context(err_context)?;
+    let pipes_to_block_or_unblock = pipes_to_block_or_unblock(running_plugin, None);
+    let plugin_render_asset =
+        PluginRenderAsset::new(plugin_id, client_id, vec![]).with_pipes(pipes_to_block_or_unblock);
+    let _ = senders
+        .send_to_plugin(PluginInstruction::UnblockCliPipes(vec![
+            plugin_render_asset,
+        ]))
+        .context("failed to unblock input pipe");
+    Ok(())
+}
