@@ -53,7 +53,7 @@ enum ExitStatus {
 
 pub struct FrameParams {
     pub focused_client: Option<ClientId>,
-    pub is_main_client: bool,
+    pub is_main_client: bool, // more accurately: is_focused_for_main_client
     pub other_focused_clients: Vec<ClientId>,
     pub style: Style,
     pub color: Option<PaletteColor>,
@@ -63,6 +63,8 @@ pub struct FrameParams {
     pub should_draw_pane_frames: bool,
     pub pane_is_floating: bool,
     pub content_offset: Offset,
+    pub mouse_is_hovering_over_pane: bool,
+    pub pane_is_selectable: bool,
 }
 
 #[derive(Default, PartialEq)]
@@ -84,6 +86,8 @@ pub struct PaneFrame {
     is_pinned: bool,
     is_floating: bool,
     content_offset: Offset,
+    mouse_is_hovering_over_pane: bool,
+    is_selectable: bool,
 }
 
 impl PaneFrame {
@@ -111,6 +115,8 @@ impl PaneFrame {
             is_pinned: false,
             is_floating: frame_params.pane_is_floating,
             content_offset: frame_params.content_offset,
+            mouse_is_hovering_over_pane: frame_params.mouse_is_hovering_over_pane,
+            is_selectable: frame_params.pane_is_selectable,
         }
     }
     pub fn is_pinned(mut self, is_pinned: bool) -> Self {
@@ -163,8 +169,9 @@ impl PaneFrame {
     ) -> Option<(Vec<TerminalCharacter>, usize)> {
         // string and length because of color
         let has_scroll = self.scroll_position.0 > 0 || self.scroll_position.1 > 0;
-        if has_scroll {
-            let pin_indication = if self.is_floating {
+        if has_scroll && self.is_selectable {
+            // TODO: don't show SCROLL at all for plugins
+            let pin_indication = if self.is_floating && self.is_selectable {
                 self.render_pinned_indication(max_length)
             } else {
                 None
@@ -189,7 +196,7 @@ impl PaneFrame {
                 (None, Some(scroll_indication)) => Some(scroll_indication),
                 _ => None,
             }
-        } else if self.is_floating {
+        } else if self.is_floating && self.is_selectable {
             self.render_pinned_indication(max_length)
         } else {
             None
@@ -285,6 +292,7 @@ impl PaneFrame {
             full_indication.push(EMPTY_TERMINAL_CHARACTER);
             full_indication.append(&mut text.clone());
             short_indication_len += 2;
+            short_indication.push(EMPTY_TERMINAL_CHARACTER);
             short_indication.append(&mut text);
         }
         if full_indication_len + 4 <= max_length {
@@ -755,6 +763,34 @@ impl PaneFrame {
         };
         Ok(res)
     }
+    fn render_mouse_shortcuts_undertitle(&self) -> Result<Vec<TerminalCharacter>> {
+        let max_undertitle_length = self.geom.cols.saturating_sub(2); // 2 for the left and right corners
+        let mut left_boundary =
+            foreground_color(self.get_corner(boundary_type::BOTTOM_LEFT), self.color);
+        let mut right_boundary =
+            foreground_color(self.get_corner(boundary_type::BOTTOM_RIGHT), self.color);
+        let res = if self.is_main_client {
+            self.empty_undertitle(max_undertitle_length)
+        } else {
+            let (mut hover_shortcuts, hover_shortcuts_len) = self.hover_shortcuts_part_full();
+            if hover_shortcuts_len <= max_undertitle_length {
+                // render exit status and tips
+                let mut padding = String::new();
+                for _ in hover_shortcuts_len..max_undertitle_length {
+                    padding.push_str(boundary_type::HORIZONTAL);
+                }
+                let mut ret = vec![];
+                ret.append(&mut left_boundary);
+                ret.append(&mut hover_shortcuts);
+                ret.append(&mut foreground_color(&padding, self.color));
+                ret.append(&mut right_boundary);
+                ret
+            } else {
+                self.empty_undertitle(max_undertitle_length)
+            }
+        };
+        Ok(res)
+    }
     pub fn clicked_on_pinned(&mut self, position: Position) -> bool {
         if self.is_floating {
             // TODO: this is not entirely accurate because our relative position calculation in
@@ -810,7 +846,16 @@ impl PaneFrame {
                     character_chunks.push(CharacterChunk::new(title, x, y));
                 } else if row == self.geom.rows - 1 {
                     // bottom row
-                    if self.exit_status.is_some() || self.is_first_run {
+                    if self.mouse_is_hovering_over_pane && !self.is_main_client {
+                        let x = self.geom.x;
+                        let y = self.geom.y + row;
+                        character_chunks.push(CharacterChunk::new(
+                            self.render_mouse_shortcuts_undertitle()
+                                .with_context(err_context)?,
+                            x,
+                            y,
+                        ));
+                    } else if self.exit_status.is_some() || self.is_first_run {
                         let x = self.geom.x;
                         let y = self.geom.y + row;
                         character_chunks.push(CharacterChunk::new(
@@ -964,6 +1009,26 @@ impl PaneFrame {
                 + break_tip.len(),
         )
     }
+    fn hover_shortcuts_part_full(&self) -> (Vec<TerminalCharacter>, usize) {
+        // (title part, length)
+        let mut hover_shortcuts = vec![];
+        let alt_click_text = " Alt <Click>";
+        let alt_click_tip = " - group,";
+        let alt_right_click_text = " Alt <Right-Click>";
+        let alt_right_click_tip = " - ungroup all ";
+
+        hover_shortcuts.append(&mut foreground_color(alt_click_text, self.color));
+        hover_shortcuts.append(&mut foreground_color(alt_click_tip, self.color));
+        hover_shortcuts.append(&mut foreground_color(alt_right_click_text, self.color));
+        hover_shortcuts.append(&mut foreground_color(alt_right_click_tip, self.color));
+        (
+            hover_shortcuts,
+            alt_click_text.chars().count()
+                + alt_click_tip.chars().count()
+                + alt_right_click_text.chars().count()
+                + alt_right_click_tip.chars().count(),
+        )
+    }
     fn empty_undertitle(&self, max_undertitle_length: usize) -> Vec<TerminalCharacter> {
         let mut left_boundary =
             foreground_color(self.get_corner(boundary_type::BOTTOM_LEFT), self.color);
