@@ -1,13 +1,16 @@
+use crate::home::default_layout_dir;
 use crate::input::actions::Action;
 use crate::input::config::ConversionError;
 use crate::input::keybinds::Keybinds;
 use crate::input::layout::{RunPlugin, SplitSize};
-use crate::shared::colors as default_colors;
+use crate::pane_size::PaneGeom;
+use crate::shared::{colors as default_colors, eightbit_to_rgb};
 use clap::ArgEnum;
 use serde::{Deserialize, Serialize};
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::fmt;
 use std::fs::Metadata;
+use std::net::IpAddr;
 use std::path::{Path, PathBuf};
 use std::str::{self, FromStr};
 use std::time::Duration;
@@ -494,6 +497,10 @@ impl KeyWithModifier {
     pub fn is_key_with_super_modifier(&self, key: BareKey) -> bool {
         self.bare_key == key && self.key_modifiers.contains(&KeyModifier::Super)
     }
+    pub fn is_cancel_key(&self) -> bool {
+        // self.bare_key == BareKey::Esc || self.is_key_with_ctrl_modifier(BareKey::Char('c'))
+        self.bare_key == BareKey::Esc
+    }
     #[cfg(not(target_family = "wasm"))]
     pub fn to_termwiz_modifiers(&self) -> Modifiers {
         let mut modifiers = Modifiers::empty();
@@ -933,6 +940,17 @@ pub enum Event {
     FailedToChangeHostFolder(Option<String>), // String -> the error we got when changing
     PastedText(String),
     ConfigWasWrittenToDisk,
+    WebServerStatus(WebServerStatus),
+    FailedToStartWebServer(String),
+    BeforeClose,
+    InterceptedKeyPress(KeyWithModifier),
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, EnumDiscriminants, ToString, Serialize, Deserialize)]
+pub enum WebServerStatus {
+    Online(String), // String -> base url
+    Offline,
+    DifferentVersion(String), // version
 }
 
 #[derive(
@@ -964,6 +982,8 @@ pub enum Permission {
     MessageAndLaunchOtherPlugins,
     Reconfigure,
     FullHdAccess,
+    StartWebServer,
+    InterceptInput,
 }
 
 impl PermissionType {
@@ -986,6 +1006,10 @@ impl PermissionType {
             },
             PermissionType::Reconfigure => "Change Zellij runtime configuration".to_owned(),
             PermissionType::FullHdAccess => "Full access to the hard-drive".to_owned(),
+            PermissionType::StartWebServer => {
+                "Start a local web server to serve Zellij sessions".to_owned()
+            },
+            PermissionType::InterceptInput => "Intercept Input (keyboard & mouse)".to_owned(),
         }
     }
 }
@@ -1092,6 +1116,48 @@ impl Default for PaletteColor {
     }
 }
 
+// these are used for the web client
+impl PaletteColor {
+    pub fn as_rgb_str(&self) -> String {
+        let (r, g, b) = match *self {
+            Self::Rgb((r, g, b)) => (r, g, b),
+            Self::EightBit(c) => eightbit_to_rgb(c),
+        };
+        format!("rgb({}, {}, {})", r, g, b)
+    }
+    pub fn from_rgb_str(rgb_str: &str) -> Self {
+        let trimmed = rgb_str.trim();
+
+        if !trimmed.starts_with("rgb(") || !trimmed.ends_with(')') {
+            return Self::default();
+        }
+
+        let inner = trimmed
+            .strip_prefix("rgb(")
+            .and_then(|s| s.strip_suffix(')'))
+            .unwrap_or("");
+
+        let parts: Vec<&str> = inner.split(',').collect();
+
+        if parts.len() != 3 {
+            return Self::default();
+        }
+
+        let mut rgb_values = [0u8; 3];
+        for (i, part) in parts.iter().enumerate() {
+            if let Some(rgb_val) = rgb_values.get_mut(i) {
+                if let Ok(parsed) = part.trim().parse::<u8>() {
+                    *rgb_val = parsed;
+                } else {
+                    return Self::default();
+                }
+            }
+        }
+
+        Self::Rgb((rgb_values[0], rgb_values[1], rgb_values[2]))
+    }
+}
+
 impl FromStr for InputMode {
     type Err = ConversionError;
 
@@ -1274,8 +1340,8 @@ pub const DEFAULT_STYLES: Styling = Styling {
     },
     frame_highlight: StyleDeclaration {
         base: PaletteColor::EightBit(default_colors::ORANGE),
-        emphasis_0: PaletteColor::EightBit(default_colors::GREEN),
-        emphasis_1: PaletteColor::EightBit(default_colors::GREEN),
+        emphasis_0: PaletteColor::EightBit(default_colors::MAGENTA),
+        emphasis_1: PaletteColor::EightBit(default_colors::PURPLE),
         emphasis_2: PaletteColor::EightBit(default_colors::GREEN),
         emphasis_3: PaletteColor::EightBit(default_colors::GREEN),
         background: PaletteColor::EightBit(default_colors::GREEN),
@@ -1432,8 +1498,8 @@ impl From<Palette> for Styling {
             },
             frame_highlight: StyleDeclaration {
                 base: palette.orange,
-                emphasis_0: palette.orange,
-                emphasis_1: palette.orange,
+                emphasis_0: palette.magenta,
+                emphasis_1: palette.purple,
                 emphasis_2: palette.orange,
                 emphasis_3: palette.orange,
                 background: Default::default(),
@@ -1508,6 +1574,14 @@ pub struct ModeInfo {
     pub session_name: Option<String>,
     pub editor: Option<PathBuf>,
     pub shell: Option<PathBuf>,
+    pub web_clients_allowed: Option<bool>,
+    pub web_sharing: Option<WebSharing>,
+    pub currently_marking_pane_group: Option<bool>,
+    pub is_web_client: Option<bool>,
+    // note: these are only the configured ip/port that will be bound if and when the server is up
+    pub web_server_ip: Option<IpAddr>,
+    pub web_server_port: Option<u16>,
+    pub web_server_capability: Option<bool>,
 }
 
 impl ModeInfo {
@@ -1554,6 +1628,8 @@ pub struct SessionInfo {
     pub is_current_session: bool,
     pub available_layouts: Vec<LayoutInfo>,
     pub plugins: BTreeMap<u32, PluginInfo>,
+    pub web_clients_allowed: bool,
+    pub web_client_count: usize,
     pub tab_history: BTreeMap<ClientId, Vec<usize>>,
 }
 
@@ -1597,6 +1673,34 @@ impl LayoutInfo {
             LayoutInfo::Stringified(_stringified) => false,
         }
     }
+    pub fn from_config(
+        layout_dir: &Option<PathBuf>,
+        default_layout: &Option<PathBuf>,
+    ) -> Option<Self> {
+        match default_layout {
+            Some(default_layout) => {
+                if default_layout.extension().is_some() || default_layout.components().count() > 1 {
+                    let Some(layout_dir) = layout_dir
+                        .as_ref()
+                        .map(|l| l.clone())
+                        .or_else(default_layout_dir)
+                    else {
+                        return None;
+                    };
+                    Some(LayoutInfo::File(
+                        layout_dir.join(default_layout).display().to_string(),
+                    ))
+                } else if default_layout.starts_with("http://")
+                    || default_layout.starts_with("https://")
+                {
+                    Some(LayoutInfo::Url(default_layout.display().to_string()))
+                } else {
+                    Some(LayoutInfo::BuiltIn(default_layout.display().to_string()))
+                }
+            },
+            None => None,
+        }
+    }
 }
 
 use std::hash::{Hash, Hasher};
@@ -1733,6 +1837,9 @@ pub struct PaneInfo {
     /// Unselectable panes are often used for UI elements that do not have direct user interaction
     /// (eg. the default `status-bar` or `tab-bar`).
     pub is_selectable: bool,
+    /// Grouped panes (usually through an explicit user action) that are staged for a bulk action
+    /// the index is kept track of in order to preserve the pane group order
+    pub index_in_pane_group: BTreeMap<ClientId, usize>,
 }
 #[derive(Debug, Clone, PartialEq, Eq, Deserialize, Serialize)]
 pub struct ClientInfo {
@@ -1763,6 +1870,7 @@ pub struct PluginIds {
     pub plugin_id: u32,
     pub zellij_pid: u32,
     pub initial_cwd: PathBuf,
+    pub client_id: ClientId,
 }
 
 /// Tag used to identify the plugin in layout and config kdl files
@@ -1870,6 +1978,7 @@ pub struct MessageToPlugin {
     /// these will only be used in case we need to launch a new plugin to send this message to,
     /// since none are running
     pub new_plugin_args: Option<NewPluginArgs>,
+    pub floating_pane_coordinates: Option<FloatingPaneCoordinates>,
 }
 
 #[derive(Debug, Default, Clone)]
@@ -1879,6 +1988,7 @@ pub struct NewPluginArgs {
     pub pane_title: Option<String>,
     pub cwd: Option<PathBuf>,
     pub skip_cache: bool,
+    pub should_focus: Option<bool>,
 }
 
 #[derive(Debug, Clone, Copy, Serialize, Deserialize, PartialEq, Eq, Hash, PartialOrd, Ord)]
@@ -1933,6 +2043,13 @@ impl MessageToPlugin {
         self.message_args = args;
         self
     }
+    pub fn with_floating_pane_coordinates(
+        mut self,
+        floating_pane_coordinates: FloatingPaneCoordinates,
+    ) -> Self {
+        self.floating_pane_coordinates = Some(floating_pane_coordinates);
+        self
+    }
     pub fn new_plugin_instance_should_float(mut self, should_float: bool) -> Self {
         let new_plugin_args = self.new_plugin_args.get_or_insert_with(Default::default);
         new_plugin_args.should_float = Some(should_float);
@@ -1961,6 +2078,11 @@ impl MessageToPlugin {
         new_plugin_args.skip_cache = true;
         self
     }
+    pub fn new_plugin_instance_should_be_focused(mut self) -> Self {
+        let new_plugin_args = self.new_plugin_args.get_or_insert_with(Default::default);
+        new_plugin_args.should_focus = Some(true);
+        self
+    }
 }
 
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
@@ -2131,6 +2253,18 @@ impl FloatingPaneCoordinates {
     }
 }
 
+impl From<PaneGeom> for FloatingPaneCoordinates {
+    fn from(pane_geom: PaneGeom) -> Self {
+        FloatingPaneCoordinates {
+            x: Some(SplitSize::Fixed(pane_geom.x)),
+            y: Some(SplitSize::Fixed(pane_geom.y)),
+            width: Some(SplitSize::Fixed(pane_geom.cols.as_usize())),
+            height: Some(SplitSize::Fixed(pane_geom.rows.as_usize())),
+            pinned: Some(pane_geom.is_pinned),
+        }
+    }
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq)]
 pub struct OriginatingPlugin {
     pub plugin_id: u32,
@@ -2148,6 +2282,77 @@ impl OriginatingPlugin {
     }
 }
 
+#[derive(ArgEnum, Deserialize, Serialize, Debug, Clone, Copy, PartialEq, Eq)]
+pub enum WebSharing {
+    #[serde(alias = "on")]
+    On,
+    #[serde(alias = "off")]
+    Off,
+    #[serde(alias = "disabled")]
+    Disabled,
+}
+
+impl Default for WebSharing {
+    fn default() -> Self {
+        Self::Off
+    }
+}
+
+impl WebSharing {
+    pub fn is_on(&self) -> bool {
+        match self {
+            WebSharing::On => true,
+            _ => false,
+        }
+    }
+    pub fn web_clients_allowed(&self) -> bool {
+        match self {
+            WebSharing::On => true,
+            _ => false,
+        }
+    }
+    pub fn sharing_is_disabled(&self) -> bool {
+        match self {
+            WebSharing::Disabled => true,
+            _ => false,
+        }
+    }
+    pub fn set_sharing(&mut self) -> bool {
+        // returns true if successfully set sharing
+        match self {
+            WebSharing::On => true,
+            WebSharing::Off => {
+                *self = WebSharing::On;
+                true
+            },
+            WebSharing::Disabled => false,
+        }
+    }
+    pub fn set_not_sharing(&mut self) -> bool {
+        // returns true if successfully set not sharing
+        match self {
+            WebSharing::On => {
+                *self = WebSharing::Off;
+                true
+            },
+            WebSharing::Off => true,
+            WebSharing::Disabled => false,
+        }
+    }
+}
+
+impl FromStr for WebSharing {
+    type Err = String;
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "On" | "on" => Ok(Self::On),
+            "Off" | "off" => Ok(Self::Off),
+            "Disabled" | "disabled" => Ok(Self::Disabled),
+            _ => Err(format!("No such option: {}", s)),
+        }
+    }
+}
+
 type Context = BTreeMap<String, String>;
 
 #[derive(Debug, Clone, EnumDiscriminants, ToString)]
@@ -2174,7 +2379,10 @@ pub enum PluginCommand {
     ShowSelf(bool), // bool - should float if hidden
     SwitchToMode(InputMode),
     NewTabsWithLayout(String), // raw kdl layout
-    NewTab,
+    NewTab {
+        name: Option<String>,
+        cwd: Option<String>,
+    },
     GoToNextTab,
     GoToPreviousTab,
     Resize(Resize),
@@ -2306,5 +2514,26 @@ pub enum PluginCommand {
     // close_plugin_after_replace
     OpenFileNearPlugin(FileToOpen, Context),
     OpenFileFloatingNearPlugin(FileToOpen, Option<FloatingPaneCoordinates>, Context),
+    StartWebServer,
+    StopWebServer,
+    ShareCurrentSession,
+    StopSharingCurrentSession,
     OpenFileInPlaceOfPlugin(FileToOpen, bool, Context), // bool -> close_plugin_after_replace
+    GroupAndUngroupPanes(Vec<PaneId>, Vec<PaneId>, bool), // panes to group, panes to ungroup,
+    // bool -> for all clients
+    HighlightAndUnhighlightPanes(Vec<PaneId>, Vec<PaneId>), // panes to highlight, panes to
+    // unhighlight
+    CloseMultiplePanes(Vec<PaneId>),
+    FloatMultiplePanes(Vec<PaneId>),
+    EmbedMultiplePanes(Vec<PaneId>),
+    QueryWebServerStatus,
+    SetSelfMouseSelectionSupport(bool),
+    GenerateWebLoginToken(Option<String>), // String -> optional token label
+    RevokeWebLoginToken(String),           // String -> token id (provided name or generated id)
+    ListWebLoginTokens,
+    RevokeAllWebLoginTokens,
+    RenameWebLoginToken(String, String), // (original_name, new_name)
+    InterceptKeyPresses,
+    ClearKeyPressesIntercepts,
+    ReplacePaneWithExistingPane(PaneId, PaneId), // (pane id to replace, pane id of existing)
 }
