@@ -180,6 +180,8 @@ pub enum Action {
     /// Open a new pane in place of the focused one, suppressing it instead
     NewInPlacePane(Option<RunCommandAction>, Option<String>), // String is an
     // optional pane
+    NewStackedPane(Option<RunCommandAction>, Option<String>), // String is an
+    // optional pane
     // name
     /// Embed focused pane in tab if floating or float focused pane if embedded
     TogglePaneEmbedOrFloating,
@@ -196,7 +198,8 @@ pub enum Action {
         Option<Vec<SwapTiledLayout>>,
         Option<Vec<SwapFloatingLayout>>,
         Option<String>,
-        bool, // should_change_focus_to_new_tab
+        bool,            // should_change_focus_to_new_tab
+        Option<PathBuf>, // cwd
     ), // the String is the tab name
     /// Do nothing.
     NoOp,
@@ -296,6 +299,8 @@ pub enum Action {
     TogglePanePinned,
     StackPanes(Vec<PaneId>),
     ChangeFloatingPaneCoordinates(PaneId, FloatingPaneCoordinates),
+    TogglePaneInGroup,
+    ToggleGroupMarking,
 }
 
 impl Action {
@@ -360,6 +365,7 @@ impl Action {
                 width,
                 height,
                 pinned,
+                stacked,
             } => {
                 let current_dir = get_current_dir();
                 // cwd should only be specified in a plugin alias if it was explicitly given to us,
@@ -441,6 +447,8 @@ impl Action {
                         )])
                     } else if in_place {
                         Ok(vec![Action::NewInPlacePane(Some(run_command_action), name)])
+                    } else if stacked {
+                        Ok(vec![Action::NewStackedPane(Some(run_command_action), name)])
                     } else {
                         Ok(vec![Action::NewTiledPane(
                             direction,
@@ -457,6 +465,8 @@ impl Action {
                         )])
                     } else if in_place {
                         Ok(vec![Action::NewInPlacePane(None, name)])
+                    } else if stacked {
+                        Ok(vec![Action::NewStackedPane(None, name)])
                     } else {
                         Ok(vec![Action::NewTiledPane(direction, None, name)])
                     }
@@ -612,6 +622,7 @@ impl Action {
                                 swap_floating_layouts.clone(),
                                 name,
                                 should_change_focus_to_new_tab,
+                                None, // the cwd is done through the layout
                             ));
                         }
                         Ok(new_tab_actions)
@@ -627,6 +638,7 @@ impl Action {
                             swap_floating_layouts,
                             name,
                             should_change_focus_to_new_tab,
+                            None, // the cwd is done through the layout
                         )])
                     }
                 } else {
@@ -638,6 +650,7 @@ impl Action {
                         None,
                         name,
                         should_change_focus_to_new_tab,
+                        cwd,
                     )])
                 }
             },
