@@ -1,11 +1,13 @@
 //! Handles cli and configuration options
 use crate::cli::Command;
-use crate::data::InputMode;
+use crate::data::{InputMode, WebSharing};
 use clap::{ArgEnum, Args};
 use serde::{Deserialize, Serialize};
 use std::path::PathBuf;
 use std::str::FromStr;
 
+use std::net::IpAddr;
+
 #[derive(Copy, Clone, Debug, PartialEq, Deserialize, Serialize, ArgEnum)]
 pub enum OnForceClose {
     #[serde(alias = "quit")]
@@ -162,6 +164,38 @@ pub struct Options {
     #[serde(default)]
     pub support_kitty_keyboard_protocol: Option<bool>,
 
+    /// Whether to make sure a local web server is running when a new Zellij session starts.
+    /// This web server will allow creating new sessions and attaching to existing ones that have
+    /// opted in to being shared in the browser.
+    ///
+    /// Note: a local web server can still be manually started from within a Zellij session or from the CLI.
+    /// If this is not desired, one can use a version of Zellij compiled without
+    /// web_server_capability
+    ///
+    /// Possible values:
+    /// - true
+    /// - false
+    /// Default: false
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub web_server: Option<bool>,
+
+    /// Whether to allow new sessions to be shared through a local web server, assuming one is
+    /// running (see the `web_server` option for more details).
+    ///
+    /// Note: if Zellij was compiled without web_server_capability, this option will be locked to
+    /// "disabled"
+    ///
+    /// Possible values:
+    /// - "on" (new sessions will allow web sharing through the local web server if it
+    /// is online)
+    /// - "off" (new sessions will not allow web sharing unless they explicitly opt-in to it)
+    /// - "disabled" (new sessions will not allow web sharing and will not be able to opt-in to it)
+    /// Default: "off"
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub web_sharing: Option<WebSharing>,
+
     /// Whether to stack panes when resizing beyond a certain size
     /// default is true
     #[clap(long, value_parser)]
@@ -179,6 +213,24 @@ pub struct Options {
     #[clap(long, value_parser)]
     #[serde(default)]
     pub show_release_notes: Option<bool>,
+
+    /// Whether to enable mouse hover effects and pane grouping functionality
+    /// default is true
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub advanced_mouse_actions: Option<bool>,
+
+    // these are intentionally excluded from the CLI options as they must be specified in the
+    // configuration file
+    pub web_server_ip: Option<IpAddr>,
+    pub web_server_port: Option<u16>,
+    pub web_server_cert: Option<PathBuf>,
+    pub web_server_key: Option<PathBuf>,
+    pub enforce_https_for_localhost: Option<bool>,
+    /// A command to run after the discovery of running commands when serializing, for the purpose
+    /// of manipulating the command (eg. with a regex) before it gets serialized
+    #[clap(long, value_parser)]
+    pub post_command_discovery_hook: Option<String>,
 }
 
 #[derive(ArgEnum, Deserialize, Serialize, Debug, Clone, Copy, PartialEq)]
@@ -257,9 +309,24 @@ impl Options {
         let support_kitty_keyboard_protocol = other
             .support_kitty_keyboard_protocol
             .or(self.support_kitty_keyboard_protocol);
+        let web_server = other.web_server.or(self.web_server);
+        let web_sharing = other.web_sharing.or(self.web_sharing);
         let stacked_resize = other.stacked_resize.or(self.stacked_resize);
         let show_startup_tips = other.show_startup_tips.or(self.show_startup_tips);
         let show_release_notes = other.show_release_notes.or(self.show_release_notes);
+        let advanced_mouse_actions = other.advanced_mouse_actions.or(self.advanced_mouse_actions);
+        let web_server_ip = other.web_server_ip.or(self.web_server_ip);
+        let web_server_port = other.web_server_port.or(self.web_server_port);
+        let web_server_cert = other
+            .web_server_cert
+            .or_else(|| self.web_server_cert.clone());
+        let web_server_key = other.web_server_key.or_else(|| self.web_server_key.clone());
+        let enforce_https_for_localhost = other
+            .enforce_https_for_localhost
+            .or(self.enforce_https_for_localhost);
+        let post_command_discovery_hook = other
+            .post_command_discovery_hook
+            .or(self.post_command_discovery_hook.clone());
 
         Options {
             simplified_ui,
@@ -289,9 +356,18 @@ impl Options {
             serialization_interval,
             disable_session_metadata,
             support_kitty_keyboard_protocol,
+            web_server,
+            web_sharing,
             stacked_resize,
             show_startup_tips,
             show_release_notes,
+            advanced_mouse_actions,
+            web_server_ip,
+            web_server_port,
+            web_server_cert,
+            web_server_key,
+            enforce_https_for_localhost,
+            post_command_discovery_hook,
         }
     }
 
@@ -350,9 +426,24 @@ impl Options {
         let support_kitty_keyboard_protocol = other
             .support_kitty_keyboard_protocol
             .or(self.support_kitty_keyboard_protocol);
+        let web_server = other.web_server.or(self.web_server);
+        let web_sharing = other.web_sharing.or(self.web_sharing);
         let stacked_resize = other.stacked_resize.or(self.stacked_resize);
         let show_startup_tips = other.show_startup_tips.or(self.show_startup_tips);
         let show_release_notes = other.show_release_notes.or(self.show_release_notes);
+        let advanced_mouse_actions = other.advanced_mouse_actions.or(self.advanced_mouse_actions);
+        let web_server_ip = other.web_server_ip.or(self.web_server_ip);
+        let web_server_port = other.web_server_port.or(self.web_server_port);
+        let web_server_cert = other
+            .web_server_cert
+            .or_else(|| self.web_server_cert.clone());
+        let web_server_key = other.web_server_key.or_else(|| self.web_server_key.clone());
+        let enforce_https_for_localhost = other
+            .enforce_https_for_localhost
+            .or(self.enforce_https_for_localhost);
+        let post_command_discovery_hook = other
+            .post_command_discovery_hook
+            .or_else(|| self.post_command_discovery_hook.clone());
 
         Options {
             simplified_ui,
@@ -382,9 +473,18 @@ impl Options {
             serialization_interval,
             disable_session_metadata,
             support_kitty_keyboard_protocol,
+            web_server,
+            web_sharing,
             stacked_resize,
             show_startup_tips,
             show_release_notes,
+            advanced_mouse_actions,
+            web_server_ip,
+            web_server_port,
+            web_server_cert,
+            web_server_key,
+            enforce_https_for_localhost,
+            post_command_discovery_hook,
         }
     }
 
@@ -450,9 +550,18 @@ impl From<CliOptions> for Options {
             styled_underlines: opts.styled_underlines,
             serialization_interval: opts.serialization_interval,
             support_kitty_keyboard_protocol: opts.support_kitty_keyboard_protocol,
+            web_server: opts.web_server,
+            web_sharing: opts.web_sharing,
             stacked_resize: opts.stacked_resize,
-            show_release_notes: opts.show_release_notes,
             show_startup_tips: opts.show_startup_tips,
+            show_release_notes: opts.show_release_notes,
+            advanced_mouse_actions: opts.advanced_mouse_actions,
+            web_server_ip: opts.web_server_ip,
+            web_server_port: opts.web_server_port,
+            web_server_cert: opts.web_server_cert,
+            web_server_key: opts.web_server_key,
+            enforce_https_for_localhost: opts.enforce_https_for_localhost,
+            post_command_discovery_hook: opts.post_command_discovery_hook,
             ..Default::default()
         }
     }
