@@ -24,6 +24,12 @@ pub mod downloader; // Requires async_std
 pub mod ipc; // Requires interprocess
 #[cfg(not(target_family = "wasm"))]
 pub mod logging; // Requires log4rs
+#[cfg(not(target_family = "wasm"))]
+pub mod sessions;
+#[cfg(all(not(target_family = "wasm"), feature = "web_server_capability"))]
+pub mod web_authentication_tokens;
+#[cfg(all(not(target_family = "wasm"), feature = "web_server_capability"))]
+pub mod web_server_commands;
 
 // TODO(hartan): Remove this re-export for the next minor release.
 pub use ::prost;
